--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 


library IEEE;
use IEEE.STD_LOGIC_1164.all;

package dcc_pack is
-- constants
constant swap_byte			: boolean := true;
constant swap_lw				: boolean := false;
constant device_ID	 		: std_logic_vector(15 downto 0) := x"DCC2";
constant CPLD_version	 	: std_logic_vector(7 downto 0) := x"01";
constant DCC_version	 	: std_logic_vector(15 downto 0) := x"3027";
constant VME_version	 	: std_logic_vector(15 downto 0) := x"0106";
constant LRB_version	 	: std_logic_vector(15 downto 0) := x"0110";
constant key				 		: std_logic_vector(31 downto 0) := x"4dcc2009";
-- use A24 to access VME chip data
-- registers on VME chip 
constant ID_addr	 			: std_logic_vector(31 downto 0) := x"00000000";
constant CRC_vme_addr	 	: std_logic_vector(31 downto 0) := x"00000004";
constant CRC_DCC_addr	 	: std_logic_vector(31 downto 0) := x"00000008";
constant CRC_LRB_addr	 	: std_logic_vector(31 downto 0) := x"0000000c";
constant VME_csr_addr 	: std_logic_vector(31 downto 0) := x"00000010"; -- VME chip control and status register
constant F_cmd_addr 		: std_logic_vector(31 downto 0) := x"00000100"; -- FLASH command register, write only
constant key_addr 			: std_logic_vector(31 downto 0) := x"00000294"; -- FLASH operation protection key register, write only
-- block RAM buffer on VME chip
constant F_wbuf_addr    : std_logic_vector(31 downto 0) := x"00000800"; -- size 0x200 FLASH write buffer
constant F_rbuf_addr    : std_logic_vector(31 downto 0) := x"00000a00"; -- size 0x200 FLASH read buffer
-- use A32 to access legacy DCC registers
---------------------------------------------------------------------------------------------------------------------------
constant CSR_addr 			: std_logic_vector(31 downto 0) := x"00000000";
constant CFG_addr 			: std_logic_vector(31 downto 0) := x"00000004";
constant MON_ctrl_addr 	: std_logic_vector(31 downto 0) := x"00000008";
constant HTR_EN_addr 		: std_logic_vector(31 downto 0) := x"0000000c";
constant TTC_id_addr 		: std_logic_vector(31 downto 0) := x"00000010";
constant TTC_i2c_addr 	: std_logic_vector(31 downto 0) := x"00000014";
constant SYNC_ctrl_addr : std_logic_vector(31 downto 0) := x"00000018";
constant SRC_id_addr 		: std_logic_vector(31 downto 0) := x"0000001c";
constant TTC_bcnt_addr 	: std_logic_vector(31 downto 0) := x"00000020";
constant TTC_cal_addr 	: std_logic_vector(31 downto 0) := x"00000024";
constant mem_status_addr: std_logic_vector(31 downto 0) := x"00000028";
constant test_ctrl_addr	: std_logic_vector(31 downto 0) := x"0000002c";
constant PAGE_addr	 		: std_logic_vector(31 downto 0) := x"00000030";
constant MON_wc_addr 		: std_logic_vector(31 downto 0) := x"00000034";
constant evn_pntr_addr	: std_logic_vector(31 downto 0) := x"00000038";
constant SL_pntr_addr		: std_logic_vector(31 downto 0) := x"0000003c";
constant sta_ctrl_addr	: std_logic_vector(31 downto 0) := x"00000100"; -- size 0x40 -- was at 0xc00
constant status_cntr_addr: std_logic_vector(31 downto 0) := x"00000140"; -- size 0x40 -- was at 0xbc0
constant mm_ctrl_addr		: std_logic_vector(31 downto 0) := x"00000180"; -- size 0x20 -- was at 0xe00
constant eb_cntr_addr		: std_logic_vector(31 downto 0) := x"00000200"; -- size 0x200
constant evn_mmc_addr		: std_logic_vector(31 downto 0) := x"00000600"; -- size 0x200
constant edc_cntr_addr	: std_logic_vector(31 downto 0) := x"00000800"; -- size 0x800
constant TTS_fifo_addr	: std_logic_vector(31 downto 0) := x"00001000"; -- size 0x1000
constant ll0_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002000";
constant ll1_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002100";
constant ll2_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002200";
constant ll3_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002300";
constant ll4_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002400";
constant edc_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002500";
constant eb_dbg_addr 		: std_logic_vector(31 downto 0) := x"00002600";
constant mem_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002700";
constant ddr_dbg_addr 	: std_logic_vector(31 downto 0) := x"00002800";
constant SL_buf_addr 		: std_logic_vector(31 downto 0) := x"00003000"; -- size 0x800
constant evn_buf_addr		: std_logic_vector(31 downto 0) := x"00004000"; -- size 0x4000
constant mem_buf_addr		: std_logic_vector(31 downto 0) := x"00008000"; -- size 0x8000
-- LRB base address
constant LRB0_base_addr : std_logic_vector(31 downto 0) := x"00010000"; -- size 0x1000
constant LRB1_base_addr : std_logic_vector(31 downto 0) := x"00011000"; -- size 0x1000
constant LRB2_base_addr : std_logic_vector(31 downto 0) := x"00012000"; -- size 0x1000
constant LRB3_base_addr : std_logic_vector(31 downto 0) := x"00013000"; -- size 0x1000
constant LRB4_base_addr : std_logic_vector(31 downto 0) := x"00014000"; -- size 0x1000
-- LRB address offset
constant LRB_VERS_addr 	: std_logic_vector(11 downto 0) := x"000";
constant LRB_PAGE_addr 	: std_logic_vector(11 downto 0) := x"004"; -- every page has 0x800 bytes memory
constant CH0_status		 	: std_logic_vector(11 downto 0) := x"008";
constant CH1_status		 	: std_logic_vector(11 downto 0) := x"00c";
constant CH2_status		 	: std_logic_vector(11 downto 0) := x"010";
-- HTR channel 0 counters
constant CH0_wc				 	: std_logic_vector(11 downto 0) := x"400";  -- total word count 8 bytes 
constant CH0_cerr				: std_logic_vector(11 downto 0) := x"408";  -- total cerr count 8 bytes 
constant CH0_uerr				: std_logic_vector(11 downto 0) := x"410";  -- total uerr count 8 bytes 
constant CH0_event_cnt	: std_logic_vector(11 downto 0) := x"418";  -- total event count 8 bytes 
constant CH0_evt_cerr 	: std_logic_vector(11 downto 0) := x"420";  -- total event count with CERR 8 bytes 
constant CH0_evt_uerr 	: std_logic_vector(11 downto 0) := x"428";  -- total event count with UERR 8 bytes 
constant CH0_evt_trunc 	: std_logic_vector(11 downto 0) := x"430";  -- total truncated event count 8 bytes 
constant CH0_evt_badid 	: std_logic_vector(11 downto 0) := x"438";  -- total event count with BADID error 8 bytes 
constant CH0_evt_crc	 	: std_logic_vector(11 downto 0) := x"440";  -- total event count with CRC error 8 bytes 
-- HTR channel 1 counters
constant CH1_wc				 	: std_logic_vector(11 downto 0) := x"500";  -- total word count 8 bytes 
constant CH1_cerr				: std_logic_vector(11 downto 0) := x"508";  -- total cerr count 8 bytes 
constant CH1_uerr				: std_logic_vector(11 downto 0) := x"510";  -- total uerr count 8 bytes 
constant CH1_event_cnt	: std_logic_vector(11 downto 0) := x"518";  -- total event count 8 bytes 
constant CH1_evt_cerr 	: std_logic_vector(11 downto 0) := x"520";  -- total event count with CERR 8 bytes 
constant CH1_evt_uerr 	: std_logic_vector(11 downto 0) := x"528";  -- total event count with UERR 8 bytes 
constant CH1_evt_trunc 	: std_logic_vector(11 downto 0) := x"530";  -- total truncated event count 8 bytes 
constant CH1_evt_badid 	: std_logic_vector(11 downto 0) := x"538";  -- total event count with BADID error 8 bytes 
constant CH1_evt_crc	 	: std_logic_vector(11 downto 0) := x"540";  -- total event count with CRC error 8 bytes 
-- HTR channel 2 counters
constant CH2_wc				 	: std_logic_vector(11 downto 0) := x"600";  -- total word count 8 bytes 
constant CH2_cerr				: std_logic_vector(11 downto 0) := x"608";  -- total cerr count 8 bytes 
constant CH2_uerr				: std_logic_vector(11 downto 0) := x"610";  -- total uerr count 8 bytes 
constant CH2_event_cnt	: std_logic_vector(11 downto 0) := x"618";  -- total event count 8 bytes 
constant CH2_evt_cerr 	: std_logic_vector(11 downto 0) := x"620";  -- total event count with CERR 8 bytes 
constant CH2_evt_uerr 	: std_logic_vector(11 downto 0) := x"628";  -- total event count with UERR 8 bytes 
constant CH2_evt_trunc 	: std_logic_vector(11 downto 0) := x"630";  -- total truncated event count 8 bytes 
constant CH2_evt_badid 	: std_logic_vector(11 downto 0) := x"638";  -- total event count with BADID error 8 bytes 
constant CH2_evt_crc	 	: std_logic_vector(11 downto 0) := x"640";  -- total event count with CRC error 8 bytes 
-- memeory window
constant LRB_MEM_addr 	: std_logic_vector(11 downto 0) := x"800"; -- memory window size 0x800
constant ethernet_addr	: std_logic_vector(31 downto 0) := x"00015000"; -- size 0x10
type td_array is array(4 downto 0) of std_logic_vector(8 downto 0);
COMPONENT deser
	PORT(
		clk : IN std_logic;
		clk_chk : IN std_logic;
		l_clk_p : IN std_logic;
		l_clk_n : IN std_logic;
		l_data_p : IN std_logic_vector(3 downto 0);
		l_data_n : IN std_logic_vector(3 downto 0);          
		polarity : IN std_logic_vector(3 downto 0);          
		l_clkout : OUT std_logic;
		strobe : OUT std_logic;
		q : OUT std_logic_vector(27 downto 0);
		deser_status : OUT std_logic_vector(11 downto 0)
		);
END COMPONENT;
COMPONENT Hamming
	PORT(
		clk : IN std_logic;
		strobe : IN std_logic;
		din : IN std_logic_vector(23 downto 0);
		cc_rxena : IN std_logic;
		cc_ecc : IN std_logic;          
		dout : OUT std_logic_vector(19 downto 0)
		);
END COMPONENT;
COMPONENT LVDS
	PORT(
		l_clk : IN std_logic;
		l_strobe : IN std_logic;
		clk : IN std_logic;
		strobe : IN std_logic;
		hq : IN std_logic_vector(19 downto 0);
		cc_rst : IN std_logic;
--		gc_block : IN std_logic;
--		gc_tstrip : IN std_logic;
--		gc_idmatch : IN std_logic;
--		gc_automon : IN std_logic;
		ch_number : IN std_logic_vector(4 downto 0);
		lfifo_ack : IN std_logic;          
--		trailer : OUT std_logic;
		lfifo_req : OUT std_logic;
		lfifo_we : OUT std_logic;
		lfifo_data : OUT std_logic_vector(32 downto 0);
--		update : OUT std_logic;
--		BADID : OUT std_logic;
		UERR : OUT std_logic;
		CERR : OUT std_logic
		);
END COMPONENT;
component refresh
		generic(sim : boolean := false; N : positive := 7);  -- default 150MHz, if clk is 200MHz, set N to 9
    Port ( clk : in std_logic;
           lock : in std_logic;
           init_done : in std_logic;
           start_init : out std_logic;
           ddr_csn : out std_logic;
           ddr_clke : out std_logic;
           tc_out : out std_logic;
           ref : out std_logic);
end component;
COMPONENT buffer_ctrl
	PORT(
		clk : IN std_logic;
		strobe : IN std_logic;
		reset : IN std_logic;
		run : IN std_logic;
		vme_rd_mem : IN std_logic;
		vme_rd_wc : IN std_logic_vector(4 downto 0);
		vme_rd_wait : OUT std_logic;
		vme_end : IN std_logic;
		vme_we_mem : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		ddr_pa : IN std_logic_vector(13 downto 0);
		addr : IN std_logic_vector(9 downto 0);
--		chip_ID : IN std_logic_vector(2 downto 0);
		lfifo_req : IN std_logic_vector(2 downto 0);
		lfifo_we : IN std_logic_vector(2 downto 0);
		lfifoA_data : IN std_logic_vector(32 downto 0);
		lfifoB_data : IN std_logic_vector(32 downto 0);
		lfifoC_data : IN std_logic_vector(32 downto 0);
		ddr_gnt : IN std_logic;
		data_fifo_afull : IN std_logic;
		rd_event : IN std_logic;
		evt_param : IN std_logic_vector(22 downto 0);
		reg_data : IN std_logic_vector(31 downto 0);
		mem_data : IN std_logic_vector(31 downto 0);          
		lfifo_ack : OUT std_logic_vector(2 downto 0);
		ddr_rqst : OUT std_logic;
		ddr_read : OUT std_logic;
		ddr_addr : OUT std_logic_vector(22 downto 0);
		ddr_length : OUT std_logic_vector(5 downto 0);
		wr_data : OUT std_logic_vector(31 downto 0);
		wr_en : OUT std_logic;
		rd_en : OUT std_logic;
		dbg_cntr0: out STD_LOGIC_VECTOR (31 downto 0);
		dbg_cntr1: out STD_LOGIC_VECTOR (31 downto 0);
		dbg_cntr2: out STD_LOGIC_VECTOR (31 downto 0);
		out_data : OUT std_logic_vector(35 downto 0)
		);
END COMPONENT;
component delayN
    generic (N: positive := 2;
	          INIT_VALUE : bit_vector := x"0000");
    port (
        c: in STD_LOGIC;
        d: in STD_LOGIC;
        ce: in STD_LOGIC;
        q: out STD_LOGIC
    );
end component;
component ddr_io
    port (
        clk: in STD_LOGIC;
        clkn: in STD_LOGIC;
        pad: inout std_logic;
        t: in STD_LOGIC;
        d0: in STD_LOGIC;
        d1: in STD_LOGIC;
        iclk: in STD_LOGIC;
        iclkn: in STD_LOGIC;
        q0: out std_logic; -- rising edge       
        q1: out std_logic  -- falling edge    
    );
end component;
COMPONENT srl_fifo
   GENERIC (N: integer := 32);
	PORT(
		rst : IN std_logic;
		din : IN std_logic_vector(N-1 downto 0);
		w_strobe : IN std_logic;
		we : IN std_logic;
		clk : IN std_logic;
		r_strobe : IN std_logic;
		re : IN std_logic;          
		dav : OUT std_logic;          
		dout : OUT std_logic_vector(N-1 downto 0);
		rp : OUT std_logic_vector(3 downto 0)
		);
END COMPONENT;
COMPONENT srl_fifo32
   GENERIC (N: integer := 32);
	PORT(
		rst : IN std_logic;
		din : IN std_logic_vector(N-1 downto 0);
		w_strobe : IN std_logic;
		we : IN std_logic;
		clk : IN std_logic;
		r_strobe : IN std_logic;
		re : IN std_logic;          
		dav : OUT std_logic;          
		dout : OUT std_logic_vector(N-1 downto 0);
		rp : OUT std_logic_vector(4 downto 0)
		);
END COMPONENT;
COMPONENT ram16XNs
   GENERIC (N : positive := 16);
	PORT(
		clk : IN std_logic;
		we : IN std_logic;
		d : IN std_logic_vector(N-1 downto 0);
		a : IN std_logic_vector(3 downto 0);          
		o : OUT std_logic_vector(N-1 downto 0)
		);
END COMPONENT;
COMPONENT ram16XNd generic( N : positive := 16);
	PORT(
		clk : IN std_logic;
		we : IN std_logic;
		d : IN std_logic_vector(N-1 downto 0);
		a : IN std_logic_vector(3 downto 0);
		dpra : IN std_logic_vector(3 downto 0);          
		spo : OUT std_logic_vector(N-1 downto 0);
		dpo : OUT std_logic_vector(N-1 downto 0)
		);
END COMPONENT;
COMPONENT vme_if
  generic (swap_byte : boolean := false; swap_lw : boolean := false);
	PORT(
		AM : IN std_logic_vector(5 downto 0);
		WRITEn : IN std_logic;
		DSn : IN std_logic_vector(1 downto 0);
		ASn : IN std_logic;
		GAn : IN std_logic_vector(4 downto 0);
		GAnP : IN std_logic;
		IACKn : IN std_logic;
		IACKINn : IN std_logic;
		SW_BAR : IN std_logic_vector(4 downto 0);
		SW_use : IN std_logic;
		clk : IN std_logic;
		inc_addr : IN std_logic;
		w_A32_ack : IN std_logic;
		we_rfifo : IN std_logic;
		r_data : IN std_logic_vector(31 downto 0);    
		A : INOUT std_logic_vector(31 downto 0);
		D : INOUT std_logic_vector(31 downto 0);      
		T_DTACK : OUT std_logic;
		DTACKn : OUT std_logic;
		BERRn : OUT std_logic;
		D_DIR : OUT std_logic;
		A_DIR : OUT std_logic;
		OEn : OUT std_logic;
		SYSFAILn : OUT std_logic;
		IACKOUTn : OUT std_logic;
		IRQ1n : OUT std_logic;
		IRQ2n : OUT std_logic;
		IRQ4n : OUT std_logic;
		IRQ7n : OUT std_logic;
		rfifo_afull : OUT std_logic;
		vme_am : OUT std_logic_vector(5 downto 0);
		vme_read : OUT std_logic;
		sel_mod : OUT std_logic;
		w_data : OUT std_logic_vector(31 downto 0);
		we_byte : OUT std_logic_vector(3 downto 0);
		addr : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT flash
  generic (swap_byte : boolean := false);
	PORT(
		clk : IN std_logic;
		vme_am : IN std_logic_vector(5 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		CPLD_str : IN std_logic;
		MQ : IN std_logic;          
		flash_wdata : OUT std_logic_vector(31 downto 0);
		flash_rdata : OUT std_logic_vector(31 downto 0);
		flash_busy : OUT std_logic;
		MD : OUT std_logic;
		MCS : OUT std_logic_vector(1 downto 0)
		);
END COMPONENT;
COMPONENT slink
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		n_strobe : IN std_logic;
		we_byte : IN std_logic_vector(3 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		en_slink : IN std_logic;
		slink64 : IN std_logic;
		slink_ctrl : IN std_logic;
		slink_din : IN std_logic_vector(35 downto 0);
		slink_we : IN std_logic;
		slink_test : IN std_logic;
		LDOWNn : IN std_logic;
		LFFn : IN std_logic;
		LRL : IN std_logic_vector(3 downto 0);
		LSF : IN std_logic_vector(3 downto 0);          
    slink_buf: out STD_LOGIC_VECTOR (31 downto 0);
    slink_pntr: out STD_LOGIC_VECTOR (31 downto 0);
		slink_send : OUT std_logic;
		slink_full : OUT std_logic;
		slink_down : OUT std_logic;
		inc_slink_wc : OUT std_logic;
		inc_slink_bc : OUT std_logic;
		inc_crc_err : OUT std_logic;
		UCLK : OUT std_logic;
		UTESTn : OUT std_logic;
		UWENn : OUT std_logic;
		URESETn : OUT std_logic;
		UCTRLn : OUT std_logic;
		UDW : OUT std_logic_vector(1 downto 0);
		UD : OUT std_logic_vector(63 downto 0);
		USF : OUT std_logic_vector(3 downto 0)
		);
END COMPONENT;
COMPONENT counters
	generic (cntr_length : integer := 12);
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		strobe : IN std_logic;
		latch_high_cnt : IN std_logic;
		uerr : IN std_logic_vector(2 downto 0);
		cerr : IN std_logic_vector(2 downto 0);
		lfifo0_data : IN std_logic_vector(32 downto 0);
		lfifo1_data : IN std_logic_vector(32 downto 0);
		lfifo2_data : IN std_logic_vector(32 downto 0);
		lfifo_we : IN std_logic_vector(2 downto 0);
		lfifo_ack : IN std_logic_vector(2 downto 0);
		addr : IN std_logic_vector(9 downto 0);          
		dout : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
COMPONENT sdram_a
	generic (sim : boolean := false);
	PORT(
		reset : IN std_logic;
		strobe : IN std_logic;
		mclk0 : IN std_logic;
		mclk90 : IN std_logic;
		mclk180 : IN std_logic;
		mclk270 : IN std_logic;
		lock : IN std_logic;
		ddr_rqst : IN std_logic;
		ddr_read : IN std_logic;
		ddr_addr : IN std_logic_vector(22 downto 0);
		ddr_length : IN std_logic_vector(5 downto 0);
		wr_data : IN std_logic_vector(31 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;    
		mdq : INOUT std_logic_vector(15 downto 0);
		dqs : INOUT std_logic_vector(1 downto 0);      
		ddr_gnt : OUT std_logic;
		rd_data : OUT std_logic_vector(31 downto 0);
		maddr : OUT std_logic_vector(12 downto 0);
		mba : OUT std_logic_vector(1 downto 0);
		dm : OUT std_logic_vector(1 downto 0);
		we_n : OUT std_logic;
		ras_n : OUT std_logic;
		cas_n : OUT std_logic;
		cs_n : OUT std_logic;
		mclke : OUT std_logic;
		mclk : OUT std_logic;
		mclk_n : OUT std_logic
		);
	END COMPONENT;
COMPONENT sdram_b
		generic (sim : boolean := false; clk_speed : positive := 1 ; clk_period : real := 10.0);
    port (
			reset: in STD_LOGIC;
			clk: in STD_LOGIC;
			mclk0: in STD_LOGIC;
			mclk90: in STD_LOGIC;
			mclk180: in STD_LOGIC;
			mclk270: in STD_LOGIC;
			n_strobe: in STD_LOGIC;
			locked: in STD_LOGIC;
		  mem_status : out STD_LOGIC_VECTOR (15 downto 0);
			rst_rfifo_data: in STD_LOGIC;
		  rfifo_data : out STD_LOGIC_VECTOR (31 downto 0);
			rfifo_dav: out STD_LOGIC;
		  ddr_gnt: out std_logic;
		  ddr_w_we: in std_logic;
		  ddr_read: in std_logic;
		  ddr_rqst: in std_logic;
			ddr_length: in STD_LOGIC_VECTOR (5 downto 0);
			ddr_addr: in STD_LOGIC_VECTOR (22 downto 0);
			ddr_w_data: in STD_LOGIC_VECTOR (35 downto 0);
--			tc : out STD_LOGIC;
			mclkio : inout STD_LOGIC;
			mdq : inout STD_LOGIC_VECTOR (15 downto 0);
			ma : out STD_LOGIC_VECTOR (12 downto 0);
			mba : out STD_LOGIC_VECTOR (1 downto 0);
			dm : out STD_LOGIC_VECTOR (1 downto 0);
			dqs : inout STD_LOGIC_VECTOR (1 downto 0);
			we_n : out STD_LOGIC;
			ras_n : out STD_LOGIC;
			cas_n : out STD_LOGIC;
			cs_n : out STD_LOGIC;
			mclke : out STD_LOGIC;
			mclk : out STD_LOGIC;
			mclk_n : out STD_LOGIC
    );
END COMPONENT;
COMPONENT ddr_rw_ctrl
	PORT(
		reset : IN std_logic;
		clk : IN std_logic;
		mclk0 : IN std_logic;
		mon_data : IN std_logic_vector(33 downto 0);
		mon_avl : IN std_logic;
		mon_we : IN std_logic;
		ddr_gnt : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		ddr_pa : IN std_logic_vector(9 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		run : IN std_logic;
		n_strobe : IN std_logic;
		mem_test: in STD_LOGIC;
		vme_cycle : IN std_logic;
		vme_read : IN std_logic;
		addr_str : IN std_logic;
		rd_length : IN std_logic_vector(3 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);          
		debug : OUT std_logic_vector(31 downto 0);
		mon_re : OUT std_logic;
		ddr_read : OUT std_logic;
		ddr_rqst : OUT std_logic;
		ddr_length : OUT std_logic_vector(5 downto 0);
		ddr_addr : OUT std_logic_vector(22 downto 0);
		ddr_w_data : OUT std_logic_vector(35 downto 0);
		ddr_we : OUT std_logic
		);
END COMPONENT;
COMPONENT monitor
	PORT(
		reset : IN std_logic;
		clk : IN std_logic;
		run : IN std_logic;
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		n_strobe : IN std_logic;
		inc_ddr_pa : IN std_logic;
		inc_crc_err : IN std_logic;
		mon_re : IN std_logic;
		ddr_pa : IN std_logic_vector(6 downto 0);
		slink_we : IN std_logic;
		slink_data : IN std_logic_vector(35 downto 0);          
		mon_evt_wc : OUT std_logic_vector(31 downto 0);
		mon_ctrl : OUT std_logic_vector(31 downto 0);
		mon_buf_empty : OUT std_logic;
		mon_buf_full : OUT std_logic;
		mon_buf_afull : OUT std_logic;
		mon_fifo_afull : OUT std_logic;
		mon_avl : OUT std_logic;
		mon_we : OUT std_logic;
		mon_data : OUT std_logic_vector(33 downto 0);
		mon_wp : OUT std_logic_vector(6 downto 0);
		inc_mon_cntr : OUT std_logic;
		catch_offset : OUT std_logic
		);
END COMPONENT;
COMPONENT vme_link
		generic (sim : boolean := false);
	PORT(
		clk : IN std_logic;
		clk180 : IN std_logic;
		clk_lock : IN std_logic;
		vmeclk : IN std_logic;
	  et_str : in  STD_LOGIC;
	  et_read : in  STD_LOGIC;
	  et_rd_length : in  STD_LOGIC_VECTOR (3 downto 0);
	  et_addr : in  STD_LOGIC_VECTOR (15 downto 0);
	  et_wdata : in  STD_LOGIC_VECTOR (31 downto 0);
	  et_sel : out  STD_LOGIC;
	  et_abort : out  STD_LOGIC;
		TXD_p : IN std_logic_vector(4 downto 0);
		TXD_n : IN std_logic_vector(4 downto 0);
		inc_addr : IN std_logic;
		r_data_val : IN std_logic;
		r_data : IN std_logic_vector(31 downto 0);          
		n_strobe : OUT std_logic;
		RXCLK_P : OUT std_logic;
		RXCLK_n : OUT std_logic;
		RXD_P : OUT std_logic_vector(4 downto 0);
		RXD_n : OUT std_logic_vector(4 downto 0);
		vme_cycle : OUT std_logic;
		LRB_ack : IN std_logic_vector(4 downto 0);
		addr_str : OUT std_logic;
		vme_read : OUT std_logic;
		mem_test : OUT std_logic;
		addr : OUT std_logic_vector(15 downto 0);
		rd_length : OUT std_logic_vector(3 downto 0);
		we_byte : OUT std_logic_vector(3 downto 0);
		w_data : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
COMPONENT LRB_link
		generic (sim : boolean := false);
	PORT(
		TCLKin : IN std_logic;
		TD_P : IN std_logic_vector(8 downto 0);
		TD_n : IN std_logic_vector(8 downto 0);
		polarity : IN std_logic_vector(11 downto 0);
		clk : IN std_logic;
		clk180 : IN std_logic;
		reset : IN std_logic;
		run : IN std_logic;
		n_strobe : IN std_logic;
		update_led : IN std_logic;
		vme_cycle : IN std_logic;
		addr_str : IN std_logic;
		vme_read : IN std_logic;
		rd_length : IN std_logic_vector(3 downto 0);
		LRB_ID : IN std_logic_vector(2 downto 0);
		led : IN std_logic_vector(4 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		lrb_test : IN std_logic;
		lrb_test_str : IN std_logic;
		lrb_test_data : IN std_logic_vector(31 downto 0);
		event_data_str : IN std_logic;
		event_data_req : IN std_logic_vector(18 downto 0);
		re_event_info : IN std_logic;
		re_event_data : IN std_logic;          
		rd_next_event : IN std_logic;          
		rCLK_P : OUT std_logic;
		rCLK_n : OUT std_logic;
		RD_P : OUT std_logic;
		RD_n : OUT std_logic;
		LRB_en : IN std_logic_vector(2 downto 0);
		LRB_ack : OUT std_logic;
		test_ack : OUT std_logic;
		reg_data_dav : OUT std_logic;
		debug : OUT std_logic_vector(31 downto 0);
		reg_data : OUT std_logic_vector(31 downto 0);
		event_info_rdy : OUT std_logic;
		event_info_val : OUT std_logic;
		event_info : OUT std_logic_vector(32 downto 0);
		event_data_rdy : OUT std_logic;
		event_data : OUT std_logic_vector(32 downto 0)
		);
END COMPONENT;
COMPONENT led_driver
	PORT(
		clk : IN std_logic;
		locked : IN std_logic;
		n_strobe : IN std_logic;
		test_led : IN std_logic;
		err_in : IN std_logic_vector(24 downto 0);
		led_in : IN std_logic_vector(24 downto 0);          
		update_led : OUT std_logic;
		led : OUT std_logic_vector(24 downto 0)
		);
END COMPONENT;
COMPONENT crc16D16
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(15 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT edc
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		addr : IN std_logic_vector(15 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		test_ctrl : IN std_logic_vector(3 downto 0);
		lrb_test : IN std_logic;
		htr_buf_addr : IN std_logic_vector(11 downto 0);
		htr_buf_length : IN std_logic_vector(8 downto 0);
		htr_buf_req : IN std_logic;
		event_data_rdy : IN std_logic_vector(4 downto 0);
		lrb0_event_data : IN std_logic_vector(32 downto 0);
		lrb1_event_data : IN std_logic_vector(32 downto 0);
		lrb2_event_data : IN std_logic_vector(32 downto 0);
		lrb3_event_data : IN std_logic_vector(32 downto 0);
		lrb4_event_data : IN std_logic_vector(32 downto 0);
		event_info_val : IN std_logic_vector(4 downto 0);
		event_info_rdy : IN std_logic_vector(4 downto 0);
		lrb0_event_info : IN std_logic_vector(32 downto 0);
		lrb1_event_info : IN std_logic_vector(32 downto 0);
		lrb2_event_info : IN std_logic_vector(32 downto 0);
		lrb3_event_info : IN std_logic_vector(32 downto 0);
		lrb4_event_info : IN std_logic_vector(32 downto 0);
		event_buf_afull : IN std_logic;
		lrb_ack : IN std_logic_vector(4 downto 0);
		htr_rd_cnt : IN std_logic_vector(3 downto 0);
		rp_addr : IN std_logic_vector(3 downto 0);
		rp : IN std_logic_vector(8 downto 0);
		htr_rd_cnt_str : IN std_logic;
		evn_ra : IN std_logic_vector(11 downto 0);
		smry_ra : IN std_logic_vector(11 downto 0);
		debug_addr : IN std_logic_vector(14 downto 0);          
		counter_addr : IN std_logic_vector(14 downto 0);          
		wp_addr : OUT std_logic_vector(3 downto 0);
		dpo_wp : OUT std_logic_vector(8 downto 0);
		htr_buf_ovf : OUT std_logic_vector(14 downto 0);
		re_event_data : OUT std_logic_vector(4 downto 0);
		rd_next_event : OUT std_logic_vector(4 downto 0);
		re_event_info : OUT std_logic_vector(4 downto 0);
		event_data_req_str : OUT std_logic_vector(4 downto 0);
		event_data_req : OUT std_logic_vector(18 downto 0);
		event_data_val : OUT std_logic;
		event_data : OUT std_logic_vector(32 downto 0);
		lrb_test_str : OUT std_logic;
		lrb_test_data : OUT std_logic_vector(31 downto 0);
		debug : OUT std_logic_vector(31 downto 0);
		summary : OUT std_logic_vector(31 downto 0);
		lrb_evn : OUT std_logic_vector(40 downto 0);
		counters : OUT std_logic_vector(31 downto 0);
		htr_state_LUT : OUT std_logic_vector(31 downto 0);
		HTR_ow : OUT std_logic;
		HTR_bz : OUT std_logic;
		HTR_state : OUT std_logic_vector(1 downto 0)
		);
END COMPONENT;
COMPONENT HTRstatus
	PORT(
		reset : IN std_logic;
		rst_cycle : IN std_logic;
		clk : IN std_logic;
		strobe : IN std_logic;
		addr : IN std_logic_vector(15 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		wr_htr_status : IN std_logic;
		htr_status_in : IN std_logic_vector(18 downto 0);          
		state_LUT : OUT std_logic_vector(31 downto 0);
		HTR_ow : OUT std_logic;
		HTR_bz : OUT std_logic;
		HTR_state : OUT std_logic_vector(1 downto 0)
		);
END COMPONENT;
COMPONENT evt_bldr
	PORT(
		reset : IN std_logic;
		reset_err : IN std_logic;
		clk : IN std_logic;
		version : IN std_logic_vector(15 downto 0);
		ttc_evcnt_reset : IN std_logic;
		event_number_avl : IN std_logic;
		event_number : IN std_logic_vector(59 downto 0);
		lrb_en : IN std_logic_vector(14 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		run : IN std_logic;
		n_strobe : IN std_logic;
		inc_l1ac : IN std_logic;
		inc_bcnterr : IN std_logic;
		inc_oc : IN std_logic;
		inc_serr : IN std_logic;
		inc_derr : IN std_logic;
		inc_rate_OFW : IN std_logic;
		inc_mon_cntr : IN std_logic;
		inc_slink_wc : IN std_logic;
		inc_slink_bc : IN std_logic;
		inc_crc_err : IN std_logic;
		htr_state : IN std_logic_vector(1 downto 0);
		wp_addr : IN std_logic_vector(3 downto 0);
		dpo_wp : IN std_logic_vector(8 downto 0);
		summary : IN std_logic_vector(31 downto 0);
		lrb_evn : IN std_logic_vector(40 downto 0);
		htr_buf_data : IN std_logic_vector(32 downto 0);
		htr_buf_val : IN std_logic;
		TTS : IN std_logic_vector(3 downto 0);          
		en_htr_ow : OUT std_logic;
		en_htr_bz : OUT std_logic;
		ovfl_warning : OUT std_logic;
		evn_fifo_full : OUT std_logic;
		err_state : OUT std_logic_vector(1 downto 0);
		status_cntr : OUT std_logic_vector(31 downto 0);
		evn_mm_cntr : OUT std_logic_vector(31 downto 0);
		mmLUT_data : OUT std_logic_vector(31 downto 0);
		evn_data : OUT std_logic_vector(31 downto 0);
		evn_pntr : OUT std_logic_vector(31 downto 0);
		counters : OUT std_logic_vector(31 downto 0);
		timeout_data : OUT std_logic_vector(31 downto 0);
		src_id_data : OUT std_logic_vector(31 downto 0);
		debug : OUT std_logic_vector(31 downto 0);
		htr_status : OUT std_logic_vector(14 downto 0);
		htr_error : OUT std_logic_vector(14 downto 0);
		rp_addr : OUT std_logic_vector(3 downto 0);
		dpo_rp : OUT std_logic_vector(8 downto 0);
		evn_ra : OUT std_logic_vector(11 downto 0);
		smry_ra : OUT std_logic_vector(11 downto 0);
		htr_rd_cnt : OUT std_logic_vector(3 downto 0);
		htr_rd_cnt_str : OUT std_logic;
		htr_buf_addr : OUT std_logic_vector(11 downto 0);
		htr_buf_length : OUT std_logic_vector(8 downto 0);
		htr_buf_req : OUT std_logic;
		slink_we : OUT std_logic;
		slink_data : OUT std_logic_vector(35 downto 0)
		);
END COMPONENT;
COMPONENT crc16D32
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(31 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT crc16D64
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(63 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT ttc_if --generic(rate_limit : std_logic_vector(9 downto 0) := "0111110010");
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		run : IN std_logic;
		n_strobe : IN std_logic;
		rst_ttc : IN std_logic;
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		en_brcst : IN std_logic;
		state : IN std_logic_vector(3 downto 0);
		evn_fifo_full : IN std_logic;
		ttc_clk : IN std_logic;
		T_ttcready : IN std_logic;
		T_l1accept : IN std_logic;
		T_brcststr : IN std_logic;
		T_brcst : IN std_logic_vector(7 downto 2);
		T_sinerrstr : IN std_logic;
		T_dberrstr : IN std_logic;
		T_evcntres : IN std_logic;
		T_bcntres : IN std_logic;
		T_bcntstr : IN std_logic;    
		T_sda : INOUT std_logic;      
		TTC_ID : OUT std_logic_vector(31 downto 0);
		TTC_BCN_off : OUT std_logic_vector(31 downto 0);
		TTC_cal_win : OUT std_logic_vector(31 downto 0);
		TTC_i2c_data : OUT std_logic_vector(31 downto 0);
		ttc_start : OUT std_logic;
		ttc_stop : OUT std_logic;
		ttc_soft_reset : OUT std_logic;
		ttc_ready : OUT std_logic;
		ttc_serr : OUT std_logic;
		ttc_derr : OUT std_logic;
		ttc_bcnt_err : OUT std_logic;
		rate_OFW : OUT std_logic;
		sync_lost : OUT std_logic;
		inc_oc : OUT std_logic;
		inc_l1ac : OUT std_logic;
		inc_bcnterr : OUT std_logic;
		inc_serr : OUT std_logic;
		inc_derr : OUT std_logic;
		ttc_evcnt_reset : OUT std_logic;
		event_number_avl : OUT std_logic;
		event_number : OUT std_logic_vector(59 downto 0);
		T_reset_b : OUT std_logic;
		T_scl : OUT std_logic;
		T_id : OUT std_logic_vector(13 downto 0)
		);
END COMPONENT;
COMPONENT ethernet
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		update_led : IN std_logic;
		vme_cycle : IN std_logic;
		vme_read : IN std_logic;
		addr_str : IN std_logic;
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		mod_sn : IN std_logic_vector(7 downto 0);
		e_intn : IN std_logic;
		addr : IN std_logic_vector(15 downto 0);
		ethernet_addr : IN std_logic_vector(15 downto 0);
		bus_gnt : IN std_logic;    
		e_data_i : IN std_logic_vector(7 downto 0);      
		e_data_o : OUT std_logic_vector(7 downto 0);      
		t_e_data : OUT std_logic;
		e_csn : OUT std_logic;
		e_wrn : OUT std_logic;
		e_rdn : OUT std_logic;
		e_addr : OUT std_logic_vector(1 downto 0);
		e_resetn : OUT std_logic;
		ethernet_data : OUT std_logic_vector(31 downto 0);
		ethernet_dav : OUT std_logic;
		bus_rqst : OUT std_logic
		);
END COMPONENT;
COMPONENT ethernetnew
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		addr_str : IN std_logic;
		mod_sn : IN std_logic_vector(7 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		r_data : IN std_logic_vector(31 downto 0);
		en_write : IN std_logic;
		ethernet_sel : IN std_logic;
		ethernet_abort : IN std_logic;
		r_data_val : IN std_logic;    
		e_data : INOUT std_logic_vector(7 downto 0);      
--		e_data_out : OUT std_logic_vector(7 downto 0);      
		e_csn : OUT std_logic;
		e_wrn : OUT std_logic;
		e_rdn : OUT std_logic;
		e_addr : OUT std_logic_vector(1 downto 0);
		ethernet_addr : OUT std_logic_vector(15 downto 0);
		ethernet_rd_length : OUT std_logic_vector(3 downto 0);
		ethernet_wdata : OUT std_logic_vector(31 downto 0);
		ethernet_read : OUT std_logic;
		ethernet_str : OUT std_logic;
		debug : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
end dcc_pack;
package body dcc_pack is
end dcc_pack;
